@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2005 NVIDIA Corporation
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *      Free Software Foundation, Inc.
- *      59 Temple Place - Suite 330
- *      Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses>.
  *
  *
  * Generate.c
@@ -604,19 +599,9 @@ typedef struct {
  * core 1.  That file contains the following copyright:
  *
  *
- *
  * mouse.py: mouse configuration data
  *
  * Copyright 1999-2002 Red Hat, Inc.
- *
- * This software may be freely redistributed under the terms of the GNU
- * library public license.
- *
- * You should have received a copy of the GNU Library Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *
  */
 
 static const MouseEntry __mice[] = {
@@ -1075,21 +1060,6 @@ typedef struct {
  * Jeremy Katz <katzj@redhat.com>
  *
  * Copyright 2002 Red Hat, Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
  */
 
 static const KeyboardEntry __keyboards[] = {
@@ -1351,7 +1321,8 @@ static int get_xserver_information(const char *versionString,
                                    int *isXorg,
                                    int *isModular,
                                    int *autoloadsGLX,
-                                   int *supportsExtensionSection)
+                                   int *supportsExtensionSection,
+                                   int *xineramaPlusCompositeWorks)
 {
 #define XSERVER_VERSION_FORMAT_1 "X Window System Version"
 #define XSERVER_VERSION_FORMAT_2 "X.Org X Server"
@@ -1366,6 +1337,7 @@ static int get_xserver_information(const char *versionString,
         *isModular = FALSE;
         *autoloadsGLX = FALSE;
         *supportsExtensionSection = FALSE;
+        *xineramaPlusCompositeWorks = FALSE;
         return TRUE;
     }
 
@@ -1430,6 +1402,17 @@ static int get_xserver_information(const char *versionString,
         *autoloadsGLX = TRUE;
     }
 
+    /*
+     * support for Xinerama and Composite at the same time works on X.Org
+     * xserver 1.15.
+     */
+
+    if ((major == 6) || (major == 7) || ((major == 1) && (minor < 15))) {
+        *xineramaPlusCompositeWorks = FALSE;
+    } else {
+        *xineramaPlusCompositeWorks = TRUE;
+    }
+
     return TRUE;
 
 } /* get_xserver_information() */
@@ -1465,6 +1448,7 @@ void xconfigGetXServerInUse(GenerateOptions *gop)
 
     gop->supports_extension_section = FALSE;
     gop->autoloads_glx = FALSE;
+    gop->xinerama_plus_composite_works = FALSE;
 
     /* run `X -version` with a PATH that hopefully includes the X binary */
 
@@ -1494,7 +1478,8 @@ void xconfigGetXServerInUse(GenerateOptions *gop)
                                         &isXorg,
                                         &dummy, /* isModular */
                                         &gop->autoloads_glx,
-                                        &gop->supports_extension_section);
+                                        &gop->supports_extension_section,
+                                        &gop->xinerama_plus_composite_works);
 
         if (found) {
             if (isXorg) {
