@@ -58,6 +58,7 @@ enum {
     NVIDIA_3DVISIONPRO_CONFIG_FILE_OPTION,
     NVIDIA_3DVISION_DISPLAY_TYPE_OPTION,
     RESTORE_ORIGINAL_BACKUP_OPTION,
+    NUM_X_SCREENS_OPTION,
 };
 
 /*
@@ -245,10 +246,6 @@ static const NVGetoptOption __options[] = {
       "update." },
 #endif
 
-    { "dynamic-twinview", XCONFIG_BOOL_VAL(DYNAMIC_TWINVIEW_BOOL_OPTION),
-      NVGETOPT_IS_BOOLEAN, NULL,
-      "Enable or disable support for dynamically configuring TwinView." },
-    
     { "preserve-driver-name", PRESERVE_DRIVER_NAME_OPTION, 0, NULL,
       "By default nvidia-xconfig changes the  display  driver  to \"nvidia\" "
       "for all configured X screens; this option preserves the existing driver "
@@ -482,13 +479,24 @@ static const NVGetoptOption __options[] = {
       XCONFIG_BOOL_VAL(SEPARATE_X_SCREENS_BOOL_OPTION),
       NVGETOPT_IS_BOOLEAN, NULL,
       "A GPU that supports multiple simultaneous display devices can either "
-      "drive these display devices in TwinView, or as separate X screens.  "
-      "When the '--separate-x-screens' option is specified, each GPU on which "
-      "an X screen is currently configured will be updated to have two X "
-      "screens configured.  The '--no-separate-x-screens' option will remove "
-      "the second configured X screen on each GPU.  Please see the NVIDIA "
-      "README description of \"Separate X Screens on One GPU\" for further "
-      "details." },
+      "drive these display devices in a single X screen, or as separate X "
+      "screens.  When the '--separate-x-screens' option is specified, each GPU "
+      "on which an X screen is currently configured will be updated to have "
+      "two or more (depending on the capabilities of that GPU) X screens "
+      "configured.  The '--no-separate-x-screens' option will remove any "
+      "extra configured X screens on each GPU.  Please see the NVIDIA README "
+      "description of \"Separate X Screens on One GPU\" for further details." },
+
+    { "x-screens-per-gpu", NUM_X_SCREENS_OPTION,
+      NVGETOPT_INTEGER_ARGUMENT, NULL,
+      "A GPU that supports multiple simultaneous display devices can either "
+      "drive these display devices in a single X screen, or as separate X "
+      "screens.  When the '--x-screens-per-gpu=<quantity>' option is "
+      "specified, each GPU on which an X screen is currently configured will "
+      "be updated to have <quantity> X screens. <quantity> has to be greater "
+      "than 0. Setting <quantity> to 1 is equivalent to specifying the "
+      "'--no-separate-x-screens' option.  Please see the NVIDIA README "
+      "description of \"Separate X Screens on One GPU\" for further details." },
 
     { "sli", SLI_OPTION,
       NVGETOPT_STRING_ARGUMENT | NVGETOPT_ALLOW_DISABLE, NULL,
@@ -502,7 +510,7 @@ static const NVGetoptOption __options[] = {
       "4 (multi-display clone mode stereo), 5 (SeeReal digital flat panel), 6 "
       "(Sharp3D digital flat panel), 7 (Arisawa/Hyundai/Zalman/Pavione/Miracube), "
       "8 (3D DLP), 9 (3D DLP INV), 10 (NVIDIA 3D VISION), "
-      "11 (NVIDIA 3D VISION PRO), 12 (HDMI 3D)." },
+      "11 (NVIDIA 3D VISION PRO), 12 (HDMI 3D), 13 (Tridelity SL)." },
 
     { "thermal-configuration-check",
       XCONFIG_BOOL_VAL(THERMAL_CONFIGURATION_CHECK_BOOL_OPTION),
@@ -640,5 +648,17 @@ static const NVGetoptOption __options[] = {
       "X configuration file at /etc/X11/xorg.conf to /etc/X11/xorg.conf."
       "nvidia-xconfig-original the first time it makes changes to that file."},
 
+    { "allow-empty-initial-configuration", XCONFIG_BOOL_VAL(ALLOW_EMPTY_INITIAL_CONFIGURATION),
+      NVGETOPT_IS_BOOLEAN, NULL, "Allow the X server to start even if no "
+      "connected display devices could be detected." },
+
+    { "delete-unused-dp12-displays", XCONFIG_BOOL_VAL(DELETE_UNUSED_DP12_DPYS),
+      NVGETOPT_IS_BOOLEAN, NULL, "Delete DisplayPort 1.2 RandR outputs that "
+      "are disconnected and not named in any MetaModes." },
+
+    { "inband-stereo-signaling", XCONFIG_BOOL_VAL(INBAND_STEREO_SIGNALING),
+      NVGETOPT_IS_BOOLEAN, NULL, "Enable or disable the "
+      "\"InbandStereoSignaling\" X configuration option." },
+
     { NULL, 0, 0, NULL, NULL },
 };
